<?php

/**
 * @file
 * Contains Drupal\snippet_manager\Form\SnippetForm.
 */

namespace Drupal\snippet_manager\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Snippet form.
 *
 * @property \Drupal\snippet_manager\SnippetInterface $entity
 */
class SnippetForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {

    $form = parent::form($form, $form_state);

    if (!$this->entity->isNew()) {
      $form['#title'] = t('Edit @label', ['@label' => $this->entity->label()]);
    }

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->label(),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => '\Drupal\snippet_manager\Entity\Snippet::load',
      ],
      '#disabled' => !$this->entity->isNew(),
    ];

    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => t('Enabled'),
      '#default_value' => $this->entity->status(),
    ];

    $code = $this->entity->getCode();

    $form['code'] = [
      '#title' => t('Code'),
      '#type' => 'text_format',
      '#default_value' => $code['value'],
      '#rows' => 10,
      '#format' => $code['format'],
      '#editor' => FALSE,
      '#required' => TRUE,
    ];

    $header = [
      $this->t('Name'),
      $this->t('Type'),
      $this->t('Operations'),
    ];

    $form['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => [],
      '#empty' => $this->t('Variables are not configured yet'),
      '#caption' => $this->t('Variables'),
    ];

    $variables = (array) $this->entity->getVariables();
    foreach ($variables as $variable_name => $variable) {

      $route_parameters = [
        'snippet' => $this->entity->id(),
        'variable' => $variable_name,
      ];

      $operation_links = [
        'edit' => [
          'title' => $this->t('Edit'),
          'url' => Url::fromRoute('snippet_manager.variable_edit_form', $route_parameters),
        ],
        'delete' => [
          'title' => $this->t('Delete'),
          'url' => Url::fromRoute('snippet_manager.variable_delete_form', $route_parameters),
        ],
      ];

      $operation_data = [
        '#type' => 'operations',
        '#links' => $operation_links,
      ];

      $variable_url = Url::fromUserInput(
        '#',
        [
          'fragment' => 'snippet-edit-form',
          'attributes' => [
            'title' => t('Insert to the textarea'),
            'class' => 'snippet-variable',
          ],
        ]
      );
      $variable_link = \Drupal::service('link_generator')->generate($variable_name, $variable_url);

      $form['table']['#rows'][$variable_name] = [
        0 => $variable_link,
        1 => $variable['plugin_id'],
        'operations' => ['data' => $operation_data],
      ];
    }

    $form['#attached']['library'][] = 'snippet_manager/snippet_manager';
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function actionsElement(array $form, FormStateInterface $form_state) {
    $element = parent::actionsElement($form, $form_state);

    if (!$this->entity->isNew()) {
      $element['add_variable'] = [
        '#type' => 'link',
        '#title' => $this->t('Add variable'),
        '#url' => Url::fromRoute('snippet_manager.variable_add_form', ['snippet' => $this->entity->id()]),
        '#attributes' => ['class' => 'button'],
        '#weight' => 5,
      ];
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {

    if ($this->entity->save()) {
      drupal_set_message($this->t('Snippet %label has been created.', [
        '%label' => $this->entity->label(),
      ]));
    }
    else {
      drupal_set_message($this->t('The %label snippet was not saved.', [
        '%label' => $this->entity->label(),
      ]));
    }

    $form_state->setRedirectUrl($this->entity->toUrl('canonical'));
  }

}
